////////////////////////////////////////////////////////////////////////////////
// TTrayIcon component
//
// Controls that allows to minimise your application in the tray-icon
//
// by Alan GARNY
//    gry@physiol.ox.ac.uk
//    http://www.physiol.ox.ac.uk/~gry
//
//  Copyright 1996-97
////////////////////////////////////////////////////////////////////////////////

#ifndef TrayIconH
#define TrayIconH

////////////////////////////////////////////////////////////////////////////////

#include <vcl\shellapi.hpp>

////////////////////////////////////////////////////////////////////////////////

enum TTrayIconOpenAppOn {ooClick, ooDblClick, ooNone};

class TTrayIcon: public TComponent
{
   private:
      // Private representation of published properties

      bool FEnabled;
      String FHint;
      TIcon *FIcon;
      TTrayIconOpenAppOn FOpenAppOn;
      TPopupMenu *FPopupMenu;

      TNotifyEvent FOnClick;
      TNotifyEvent FOnDblClick;
      TMouseEvent FOnMouseDown;
      TMouseEvent FOnMouseMove;
      TMouseEvent FOnMouseUp;

      // Properties used for internal purpose

      HWND HandleWnd;   // Handler for the window

      TNotifyIconData IconData;

      TForm *Owner;
      HWND OwnerHandle;   // = Owner->Handle. Useful to have it, since we lose
                          // its original value when destroying the form, which
                          // results in some strange behaviours... :-(

      int (__stdcall *OldHookFormProc) ();
      int (__stdcall *NewHookFormProc) ();

      bool SetupProperly;

      bool IconInserted;

      // Methods to modify the different published properties

      void __fastcall SetEnabled(const bool aValue);
      void __fastcall SetHint(const String aValue);
      void __fastcall SetIcon(TIcon *aValue);
      void __fastcall SetOpenAppOn(const TTrayIconOpenAppOn aValue);
      void __fastcall SetPopupMenu(TPopupMenu *aValue);

      // Methods used for internal purpose

      void __fastcall HookForm();
      void __fastcall UnhookForm();

      void __fastcall HookFormProc(TMessage &aMsg);

      void __fastcall FillIconDataStructure();

      void __fastcall InsertIcon();
      void __fastcall RemoveIcon();

      void __fastcall MinimizeApp();
      void __fastcall RestoreApp();

      // Methods used for internal purpose

      void __fastcall WndProc(TMessage &aMessage);

      inline bool __fastcall CanPlayWithTrayIcon();

      // Overridden methods

      virtual void __fastcall Loaded();

   public:
      // Constructor & Destructor

      virtual __fastcall TTrayIcon(TComponent *aOwner);
      virtual __fastcall ~TTrayIcon();

      void __fastcall IconModified();
   __published:
      // Published properties

      __property bool Enabled = {read=FEnabled, write=SetEnabled, default=true};
      __property String Hint = {read=FHint, write=SetHint, nodefault};
      __property TIcon *Icon = {read=FIcon, write=SetIcon, nodefault};
      __property TTrayIconOpenAppOn OpenAppOn = {read=FOpenAppOn, write=SetOpenAppOn, default=ooDblClick};
      __property TPopupMenu *PopupMenu = {read=FPopupMenu, write=FPopupMenu, nodefault};

      // Available events

      __property TNotifyEvent OnClick = {read=FOnClick, write=FOnClick};
      __property TNotifyEvent OnDblClick = {read=FOnDblClick, write=FOnDblClick};
      __property TMouseEvent OnMouseDown = {read=FOnMouseDown, write=FOnMouseDown};
      __property TMouseEvent OnMouseMove = {read=FOnMouseMove, write=FOnMouseMove};
      __property TMouseEvent OnMouseUp = {read=FOnMouseUp, write=FOnMouseUp};
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////

